<?php

// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Forms\Navigation;

class IcingadbHostActionForm extends ActionForm
{
    protected $restriction = 'icingadb/filter/hosts';
}
