<?php

// SPDX-FileCopyrightText: 2019 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Icingadb\Common;

class Icons
{
    public const COMMENT = 'comment';

    public const HOST_DOWN = 'sitemap';

    public const UNREACHABLE = 'sitemap';

    public const IN_DOWNTIME = 'plug';

    public const IS_ACKNOWLEDGED = 'check';

    public const IS_FLAPPING = 'bolt';

    public const IS_PERSISTENT = 'thumbtack';

    public const NOTIFICATION = 'bell';

    public const REMOVE = 'trash';

    public const NOTIFICATIONS_DISABLED = 'bell-slash';

    public const ACTIVE_CHECKS_DISABLED = 'eye-slash';

    public const USER = 'user';

    public const USERGROUP = 'users';

    public const WARNING = 'exclamation-triangle';
}
