// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

// Style

.item-layout {
  &.overdue {
    background-color: @gray-lighter;
  }

  &.overdue header > *:not(time),
  &.overdue .caption {
    opacity: 0.6;
  }

  &.overdue time {
    .rounded-corners();
    background-color: @color-critical;
    color: @text-color-on-icinga-blue;
  }

  .title {
    .state-text {
      color: @text-color;
      text-transform: uppercase;
    }
  }

  footer {
    .status-icons {
      color: @text-color-light;

      .icon {
        opacity: .5;
      }
    }
  }
}

// Layout

.item-layout {
  &.overdue time {
    margin-right: -.5em;
    padding: 0 0.5em;
  }

  .visual .check-attempt {
    margin-top: .5em;
  }

  .caption {
    &.plugin-output, .plugin-output {
      font-size: 11/12em;
      line-height: 1.5*12/11em;
    }
  }

  &.minimal-item-layout,
  &.default-item-layout {
    .caption .plugin-output {
      .text-ellipsis();
    }
  }

  &.minimal-item-layout.hostgroup .extended-info {
    > :not(:last-child) {
      margin-right: 1em;
    }
  }

  footer {
    .status-icons {
      display: flex;
      align-items: center;
    }

    .performance-data {
      margin-left: auto;

      .inline-pie {
        display: inline-block;
        line-height: 1.5*.857em;
        height: 1em;
        width: 1em;

        &:not(:last-child) {
          margin-right: .209em;
        }
      }
    }
  }
}
