// SPDX-FileCopyrightText: 2024 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.item-layout.redundancy-group {
  .caption {
    display: flex;
    justify-content: flex-end;
  }
}

.default-item-layout.redundancy-group {
  .title > .subject {
    margin-right: .28125em;
  }

  .caption .object-statistics {
    font-size: 0.75em;
  }
}
