// SPDX-FileCopyrightText: 2020 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

.donut-container {
  .card();

  h2 {
    margin: 0;
  }

  .state-badges {
    text-align: center;
  }

  &:not(:last-of-type) {
    margin-right: 1em;
    margin-bottom: 1em;
  }

  .donut {
    margin: 0 auto;
  }
}

#layout.minimal-layout .donut-container {
  width: 100%;
}
